@@ -107,6 +107,10 @@
 # endif
 #endif
 
+#ifdef OF_MORPHOS
+# include <dos/dostags.h>
+#endif
+
 #if defined(OF_WINDOWS) || defined(OF_AMIGAOS)
 typedef struct {
 	OFStreamOffset st_size;
@@ -135,7 +139,7 @@ static OFMutex *passwdMutex;
 static void
 releasePasswdMutex(void)
 {
-	[passwdMutex release];
+	objc_release(passwdMutex);
 }
 #endif
 #if defined(OF_HAVE_THREADS) && !defined(__GLIBC__) && !defined(OF_WINDOWS)
@@ -144,7 +148,7 @@ static OFMutex *readdirMutex;
 static void
 releaseReaddirMutex(void)
 {
-	[readdirMutex release];
+	objc_release(readdirMutex);
 }
 #endif
 
@@ -312,7 +316,18 @@ statWrapper(OFString *path, Stat *buffer)
 		return lastError();
 
 # if defined(OF_MORPHOS)
-	if (!Examine64(lock, &fib, TAG_DONE)) {
+	struct TagItem tags[] = {
+#  ifdef EX64TAG_PosixDate
+		{ EX64TAG_PosixDate, TRUE },
+#  endif
+		{ TAG_DONE, 0 }
+	};
+
+#  ifdef FIBEXTF_POSIXDATE
+	fib.fib_ActExtFlags = 0;
+#  endif
+
+	if (!Examine64(lock, &fib, tags)) {
 # elif defined(OF_AMIGAOS4)
 	if ((ed = ExamineObjectTags(EX_FileLockInput, lock, TAG_END)) == NULL) {
 # else
@@ -338,25 +353,35 @@ statWrapper(OFString *path, Stat *buffer)
 	buffer->st_mode = (fib.fib_DirEntryType > 0 ? S_IFDIR : S_IFREG);
 # endif
 
-	timeInterval = 252460800;	/* 1978-01-01 */
+# if defined(OF_MORPHOS) && defined(FIBEXTF_POSIXDATE)
+	if (fib.fib_ActExtFlags & FIBEXTF_POSIXDATE)
+		timeInterval = fib.fib_PosixDate.pds_Sec;
+	else {
+# endif
 
-	locale = OpenLocale(NULL);
-	/*
-	 * FIXME: This does not take DST into account. But unfortunately, there
-	 * is no way to figure out if DST was in effect when the file was
-	 * modified.
-	 */
-	timeInterval += locale->loc_GMTOffset * 60.0;
-	CloseLocale(locale);
+		timeInterval = 252460800;	/* 1978-01-01 */
+
+		locale = OpenLocale(NULL);
+		/*
+		 * FIXME: This does not take DST into account. But
+		 * unfortunately, there is no way to figure out if DST was in
+		 * effect when the file was modified.
+		 */
+		timeInterval += locale->loc_GMTOffset * 60.0;
+		CloseLocale(locale);
 
 # ifdef OF_AMIGAOS4
-	date = &ed->Date;
+		date = &ed->Date;
 # else
-	date = &fib.fib_Date;
+		date = &fib.fib_Date;
+# endif
+		timeInterval += date->ds_Days * 86400.0;
+		timeInterval += date->ds_Minute * 60.0;
+		timeInterval +=
+		    date->ds_Tick / (OFTimeInterval)TICKS_PER_SECOND;
+# if defined(OF_MORPHOS) && defined(FIBEXTF_POSIXDATE)
+	}
 # endif
-	timeInterval += date->ds_Days * 86400.0;
-	timeInterval += date->ds_Minute * 60.0;
-	timeInterval += date->ds_Tick / (OFTimeInterval)TICKS_PER_SECOND;
 
 	buffer->st_atime = buffer->st_mtime = buffer->st_ctime = timeInterval;
 
@@ -810,7 +835,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 			@try {
 				[names addObject: name];
 			} @finally {
-				[name release];
+				objc_release(name);
 			}
 		}
 	} @finally {
@@ -890,11 +915,11 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 							     errNo: e.errNo];
 	}
 
-	[file retain];
+	objc_retain(file);
 
 	objc_autoreleasePoolPop(pool);
 
-	return [file autorelease];
+	return objc_autoreleaseReturnValue(file);
 }
 
 - (OFFileAttributes)attributesOfItemAtIRI: (OFIRI *)IRI
@@ -957,8 +982,9 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 
 	if (lastAccessDate == nil)
 		lastAccessDate = modificationDate;
-	if (modificationDate == nil)
+	if (modificationDate == nil) {
 		modificationDate = lastAccessDate;
+	}
 
 #if defined(OF_WINDOWS)
 	FILETIME accessTime = timeIntervalToFiletime(
@@ -1035,28 +1061,74 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		     failedAttribute: attributeKey
 			       errNo: lastError()];
 #else
-	OFTimeInterval lastAccessTime = lastAccessDate.timeIntervalSince1970;
-	OFTimeInterval modificationTime =
-	    modificationDate.timeIntervalSince1970;
-	struct timeval times[2] = {
-		{
-			.tv_sec = (time_t)lastAccessTime,
-			.tv_usec = (int)((lastAccessTime -
-			    (time_t)lastAccessTime) * 1000000)
-		},
-		{
-			.tv_sec = (time_t)modificationTime,
-			.tv_usec = (int)((modificationTime -
-			    (time_t)modificationTime) * 1000000)
-		},
-	};
+# ifdef HAVE_UTIMENSAT
+#  if defined(OF_MACOS) || defined(OF_IOS)
+	if (@available(macOS 10.13, iOS 11, *)) {
+#  endif
+		OFTimeInterval lastAccessTime =
+		    lastAccessDate.timeIntervalSince1970;
+		OFTimeInterval modificationTime =
+		    modificationDate.timeIntervalSince1970;
+		struct timespec times[2] = {
+			{
+				.tv_sec = (time_t)lastAccessTime,
+				.tv_nsec = (int)((lastAccessTime -
+				    (time_t)lastAccessTime) * 1000000000)
+			},
+			{
+				.tv_sec = (time_t)modificationTime,
+				.tv_nsec = (long)((modificationTime -
+				    (time_t)modificationTime) * 1000000000)
+			},
+		};
 
-	if (utimes([path cStringWithEncoding: [OFLocale encoding]], times) != 0)
-		@throw [OFSetItemAttributesFailedException
-		    exceptionWithIRI: IRI
-			  attributes: attributes
-		     failedAttribute: attributeKey
-			       errNo: errno];
+		if (utimensat(AT_FDCWD, [path cStringWithEncoding:
+		    [OFLocale encoding]], times, AT_SYMLINK_NOFOLLOW) != 0) {
+			@throw [OFSetItemAttributesFailedException
+				exceptionWithIRI: IRI
+				      attributes: attributes
+				 failedAttribute: attributeKey
+					   errNo: errno];
+		}
+#  if defined(OF_MACOS) || defined(OF_IOS)
+	} else {
+#  endif
+# endif
+# if !defined(HAVE_UTIMENSAT) || defined(OF_MACOS) || defined(OF_IOS)
+		OFTimeInterval lastAccessTime =
+		    lastAccessDate.timeIntervalSince1970;
+		OFTimeInterval modificationTime =
+		    modificationDate.timeIntervalSince1970;
+		struct timeval times[2] = {
+			{
+				.tv_sec = (time_t)lastAccessTime,
+				.tv_usec = (int)((lastAccessTime -
+					(time_t)lastAccessTime) * 1000000)
+			},
+			{
+				.tv_sec = (time_t)modificationTime,
+				.tv_usec = (int)((modificationTime -
+					(time_t)modificationTime) * 1000000)
+			},
+		};
+
+# ifdef HAVE_LUTIMES
+		if (lutimes([path cStringWithEncoding: [OFLocale encoding]],
+		    times) != 0) {
+# else
+		if (utimes([path cStringWithEncoding: [OFLocale encoding]],
+		    times) != 0) {
+# endif
+			@throw [OFSetItemAttributesFailedException
+			    exceptionWithIRI: IRI
+				  attributes: attributes
+			     failedAttribute: attributeKey
+				       errNo: errno];
+		}
+#  if defined(HAVE_UTIMENSAT) && (defined(OF_MACOS) || defined(OF_IOS))
+	}
+#  endif
+# endif
 #endif
 }
 
@@ -1342,7 +1414,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 					[IRIs addObject: [IRI
 					    IRIByAppendingPathComponent: file]];
 				} @finally {
-					[file release];
+					objc_release(file);
 				}
 			} while (FindNextFileW(handle, &fd));
 
@@ -1381,7 +1453,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 					[IRIs addObject: [IRI
 					    IRIByAppendingPathComponent: file]];
 				} @finally {
-					[file release];
+					objc_release(file);
 				}
 			} while (FindNextFileA(handle, &fd));
 
@@ -1428,7 +1500,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 					[IRIs addObject: [IRI
 					    IRIByAppendingPathComponent: file]];
 				} @finally {
-					[file release];
+					objc_release(file);
 				}
 			}
 		} @finally {
@@ -1451,7 +1523,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 				[IRIs addObject:
 				    [IRI IRIByAppendingPathComponent: file]];
 			} @finally {
-				[file release];
+				objc_release(file);
 			}
 		}
 # endif
@@ -1507,7 +1579,7 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 				[IRIs addObject:
 				    [IRI IRIByAppendingPathComponent: file]];
 			} @finally {
-				[file release];
+				objc_release(file);
 			}
 		}
 	} @finally {
@@ -1907,15 +1979,15 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 		*type = nil;
 # endif
 
-	[*data retain];
+	objc_retain(*data);
 	if (type != NULL)
-		[*type retain];
+		objc_retain(*type);
 
 	objc_autoreleasePoolPop(pool);
 
-	[*data autorelease];
+	objc_autorelease(*data);
 	if (type != NULL)
-		[*type autorelease];
+		objc_autorelease(*type);
 }
 
 - (void)setExtendedAttributeData: (OFData *)data
@@ -1978,15 +2050,13 @@ setExtendedAttributes(OFMutableFileAttributes attributes, OFIRI *IRI)
 	}
 # elif defined(OF_HAIKU)
 	const char *cName = [name cStringWithEncoding: encoding];
-	unsigned long long typeInt;
+	unsigned int typeInt;
 	int fd;
 
 	if (type != nil && ![type isKindOfClass: [OFNumber class]])
 		@throw [OFInvalidArgumentException exception];
 
-	typeInt = (type != nil ? [type unsignedLongLongValue] : 0);
-	if (typeInt > UINT32_MAX)
-		@throw [OFInvalidArgumentException exception];
+	typeInt = (type != nil ? [type unsignedIntValue] : 0);
 
 	if (size > SSIZE_MAX)
 		@throw [OFOutOfRangeException exception];
