@@ -51,6 +51,8 @@ indent(OFString *string)
 static void
 setPermissions(OFString *path, OFLHAArchiveEntry *entry)
 {
+	[app quarantineFile: path];
+
 #ifdef OF_FILE_MANAGER_SUPPORTS_PERMISSIONS
 	OFNumber *POSIXPermissions = entry.POSIXPermissions;
 
@@ -67,8 +69,6 @@ setPermissions(OFString *path, OFLHAArchiveEntry *entry)
 						 ofItemAtPath: path];
 	}
 #endif
-
-	[app quarantineFile: path];
 }
 
 static void
@@ -98,10 +98,11 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 			  mode: (OFString *)mode
 		      encoding: (OFStringEncoding)encoding
 {
-	return [[[self alloc] initWithIRI: IRI
-				   stream: stream
-				     mode: mode
-				 encoding: encoding] autorelease];
+	return objc_autoreleaseReturnValue(
+	    [[self alloc] initWithIRI: IRI
+			       stream: stream
+				 mode: mode
+			     encoding: encoding]);
 }
 
 - (instancetype)initWithIRI: (OFIRI *)IRI
@@ -118,7 +119,7 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 		if (encoding != OFStringEncodingAutodetect)
 			_archive.encoding = encoding;
 	} @catch (id e) {
-		[self release];
+		objc_release(self);
 		@throw e;
 	}
 
@@ -127,7 +128,7 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 
 - (void)dealloc
 {
-	[_archive release];
+	objc_release(_archive);
 
 	[super dealloc];
 }
@@ -139,6 +140,8 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 	while ((entry = [_archive nextEntry]) != nil) {
 		void *pool = objc_autoreleasePoolPush();
 
+		[app checkForCancellation];
+
 		[OFStdOut writeLine: entry.fileName];
 
 		if (app->_outputLevel >= 1) {
@@ -287,6 +290,8 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 		unsigned long long written = 0, size = entry.uncompressedSize;
 		int8_t percent = -1, newPercent;
 
+		[app checkForCancellation];
+
 		if (!all && ![files containsObject: fileName])
 			continue;
 
@@ -304,7 +309,7 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 		}
 
 		if (app->_outputLevel >= 0)
-			[OFStdOut writeString: OF_LOCALIZED(@"extracting_file",
+			[OFStdErr writeString: OF_LOCALIZED(@"extracting_file",
 			    @"Extracting %[file]...",
 			    @"file", fileName)];
 
@@ -322,8 +327,8 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 					   secondObject: entry]];
 
 			if (app->_outputLevel >= 0) {
-				[OFStdOut writeString: @"\r"];
-				[OFStdOut writeLine: OF_LOCALIZED(
+				[OFStdErr writeString: @"\r"];
+				[OFStdErr writeLine: OF_LOCALIZED(
 				    @"extracting_file_done",
 				    @"Extracting %[file]... done",
 				    @"file", fileName)];
@@ -345,7 +350,11 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 		setPermissions(outFileName, entry);
 
 		while (!stream.atEndOfStream) {
-			ssize_t length = [app copyBlockFromStream: stream
+			ssize_t length;
+
+			[app checkForCancellation];
+
+			length = [app copyBlockFromStream: stream
 							 toStream: output
 							 fileName: fileName];
 
@@ -365,8 +374,8 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 				percentString = [OFString stringWithFormat:
 				    @"%3u", percent];
 
-				[OFStdOut writeString: @"\r"];
-				[OFStdOut writeString: OF_LOCALIZED(
+				[OFStdErr writeString: @"\r"];
+				[OFStdErr writeString: OF_LOCALIZED(
 				    @"extracting_file_percent",
 				    @"Extracting %[file]... %[percent]%",
 				    @"file", fileName,
@@ -378,8 +387,8 @@ setModificationDate(OFString *path, OFLHAArchiveEntry *entry)
 		setModificationDate(outFileName, entry);
 
 		if (app->_outputLevel >= 0) {
-			[OFStdOut writeString: @"\r"];
-			[OFStdOut writeLine: OF_LOCALIZED(
+			[OFStdErr writeString: @"\r"];
+			[OFStdErr writeLine: OF_LOCALIZED(
 			    @"extracting_file_done",
 			    @"Extracting %[file]... done",
 			    @"file", fileName)];
@@ -421,15 +430,21 @@ outer_loop_end:
 		OFString *fileName = entry.fileName;
 		OFStream *stream;
 
+		[app checkForCancellation];
+
 		if (![files containsObject: fileName])
 			continue;
 
 		stream = [_archive streamForReadingCurrentEntry];
 
 		while (!stream.atEndOfStream) {
-			ssize_t length = [app copyBlockFromStream: stream
-							 toStream: OFStdOut
-							 fileName: fileName];
+			ssize_t length;
+
+			[app checkForCancellation];
+
+			length = [app copyBlockFromStream: stream
+						 toStream: OFStdOut
+						 fileName: fileName];
 
 			if (length < 0) {
 				app->_exitStatus = 1;
@@ -464,8 +479,10 @@ outer_loop_end:
 		OFMutableLHAArchiveEntry *entry;
 		OFStream *output;
 
+		[app checkForCancellation];
+
 		if (app->_outputLevel >= 0)
-			[OFStdOut writeString: OF_LOCALIZED(@"adding_file",
+			[OFStdErr writeString: OF_LOCALIZED(@"adding_file",
 			    @"Adding %[file]...",
 			    @"file", fileName)];
 
@@ -509,10 +526,13 @@ outer_loop_end:
 							mode: @"r"];
 
 			while (!input.atEndOfStream) {
-				ssize_t length = [app
-				    copyBlockFromStream: input
-					       toStream: output
-					       fileName: fileName];
+				ssize_t length;
+
+				[app checkForCancellation];
+
+				length = [app copyBlockFromStream: input
+							 toStream: output
+							 fileName: fileName];
 
 				if (length < 0) {
 					app->_exitStatus = 1;
@@ -531,8 +551,8 @@ outer_loop_end:
 					percentString = [OFString
 					    stringWithFormat: @"%3u", percent];
 
-					[OFStdOut writeString: @"\r"];
-					[OFStdOut writeString: OF_LOCALIZED(
+					[OFStdErr writeString: @"\r"];
+					[OFStdErr writeString: OF_LOCALIZED(
 					    @"adding_file_percent",
 					    @"Adding %[file]... %[percent]%",
 					    @"file", fileName,
@@ -542,8 +562,8 @@ outer_loop_end:
 		}
 
 		if (app->_outputLevel >= 0) {
-			[OFStdOut writeString: @"\r"];
-			[OFStdOut writeLine: OF_LOCALIZED(
+			[OFStdErr writeString: @"\r"];
+			[OFStdErr writeLine: OF_LOCALIZED(
 			    @"adding_file_done",
 			    @"Adding %[file]... done",
 			    @"file", fileName)];
